//package micsa.fun;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


//import ChromosomeNamesComparator;

public class Summary {
    private static Comparator<String> byChromNames = new ChromosomeNamesComparator();

	public static void createSummary(double ratio, File outputDir,String chipOutFilename,String controlOutFilename,String summaryFilename) {
    		Hashtable <String,ArrayList <Float>> chipHash = readToHash(new File(outputDir,chipOutFilename));
    		Hashtable <String,ArrayList <Float>> controlHash = readToHash(new File(outputDir,controlOutFilename));

    		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File (outputDir, summaryFilename)));
			ArrayList<String> chroms = new ArrayList<String>(chipHash.keySet());
			Collections.sort(chroms, byChromNames);
			for (String chrom : chroms) {
						bw.write("Info:    Current chromosome : "+chrom+"\n") ;
						for (int i = 0; i<chipHash.get(chrom).size(); i++) {
							bw.write("Info:    "+i+"	"+i+"	"+ chipHash.get(chrom).get(i)+ "	") ;
							if(!controlHash.containsKey(chrom)) {
								controlHash.put(chrom, new ArrayList<Float>());
							}

							for (int j = controlHash.get(chrom).size(); j<=i; j++)
								controlHash.get(chrom).add(j, 0f);

							bw.write(controlHash.get(chrom).get(i)+"	"+controlHash.get(chrom).get(i)*ratio+"	") ;
							Double diff = chipHash.get(chrom).get(i)-controlHash.get(chrom).get(i)*ratio;
							bw.write(diff.toString()) ;
							bw.newLine();
						}
			}
			bw.close() 	;
	    	} catch (IOException e) {
				e.printStackTrace();
		}
	}
	public static void createSummary(double ratio, String chipOutFilename,String controlOutFilename,String summaryFilename) {
    		System.out.println("..reading \n"+chipOutFilename+"\n and \n"+controlOutFilename+ "\n");

    		Hashtable <String,ArrayList <Float>> chipHash = readToHash(new File(chipOutFilename));
    		Hashtable <String,ArrayList <Float>> controlHash = readToHash(new File(controlOutFilename));

    		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File (summaryFilename)));
			ArrayList<String> chroms = new ArrayList<String>(chipHash.keySet());
			Collections.sort(chroms, byChromNames);
			for (String chrom : chroms) {
						bw.write("Info:    Current chromosome : "+chrom+"\n") ;
						for (int i = 0; i<chipHash.get(chrom).size(); i++) {
							bw.write("Info:    "+i+"	"+i+"	"+ chipHash.get(chrom).get(i)+ "	") ;
							if(!controlHash.containsKey(chrom)) {
								controlHash.put(chrom, new ArrayList<Float>());
							}

							for (int j = controlHash.get(chrom).size(); j<=i; j++)
								controlHash.get(chrom).add(j, 0f);

							bw.write(controlHash.get(chrom).get(i)+"	"+controlHash.get(chrom).get(i)*ratio+"	") ;
							Double diff = chipHash.get(chrom).get(i)-controlHash.get(chrom).get(i)*ratio;
							bw.write(diff.toString()) ;
							bw.newLine();
						}
			}
			bw.close() 	;
	    	} catch (IOException e) {
				e.printStackTrace();
		}
	}

	private static Hashtable <String,ArrayList <Float>> readToHash(File file) {
    	Hashtable <String,ArrayList <Float>> hash = new Hashtable <String,ArrayList <Float>>();
   		Pattern pattern = Pattern.compile( ".*?(c?h?r?[0-9XYMTWZ]+)\\s\\d+\\s\\d+\\s\\d+\\s([0-9]+\\.[0-9]+).*");
		String s;
		int maximalIndex =0;
    	try {
    		BufferedReader br = new BufferedReader(new FileReader(file));

			while ((s = br.readLine())!=null) {
				   //System.out.println(s);
				   s = s.replace(".fa", "");
				   Matcher matcher = pattern.matcher( s );
				   if (matcher.find()) {
						String chrom = matcher.group(1);
						Float score = Float.parseFloat(matcher.group(2));
						Integer index = Math.round(score);
						maximalIndex = Math.max(maximalIndex, index);
						if (!hash.containsKey(chrom)) {
							hash.put(chrom, new ArrayList <Float>());
							//hash.get(chrom).add(0, 0f);
						}

						for (int i = hash.get(chrom).size(); i<=index; i++)
							hash.get(chrom).add(i,0f);


						for (int i = 0; i<= index; i++) {		//!!starting from zero!! check if it is good TODO
							Float pv = hash.get(chrom).get(i);
							if (pv == null)
								pv = 0f;
							hash.get(chrom).set(i, pv+1);
						}
				   }
			}
			br.close();
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		for (String chrom : hash.keySet()) {
			if (hash.get(chrom).size()<maximalIndex) {
				for (int i = hash.get(chrom).size(); i<=maximalIndex; i++)
					hash.get(chrom).add(i,0f);
			}
		}

		return hash;
	}
	public static void main(String[] args) throws IOException {
		String usage = "java -Xmx1565m Summary -f <FindPeaks output file for ChIP> -c <FindPeaks output file for Control> -r <minimal ratio between depths of peaks in ChIP and Normalized Control>";

		if (args.length < 6) {
			System.err.println("Too few arguments\n");
			System.err.println(usage);
			System.exit(1);
		}

		double ratio = 0;
		//File outputDir = new File (".");
		String chipOutFilename = null;
		String controlOutFilename = null;
		String summaryFilename = "FindPeaksSummary.txt";

		System.out.println("number of arguments: "+args.length);

		int currentIndex = 0;

		while (currentIndex<args.length) {

			System.out.println("arg "+currentIndex+": "+args[currentIndex]);

			if (args[currentIndex].compareTo("-f")==0) {
				currentIndex++;
				chipOutFilename = args[currentIndex];
				System.out.println("arg "+currentIndex+": "+args[currentIndex]);
			}
			else if (args[currentIndex].compareTo("-c")==0) {
				currentIndex++;
				controlOutFilename = args[currentIndex];
				System.out.println("arg "+currentIndex+": "+args[currentIndex]);
			}

			else if (args[currentIndex].compareTo("-r")==0) {
				currentIndex++;
				ratio = Double.parseDouble(args[currentIndex]);
				System.out.println("arg "+currentIndex+": "+args[currentIndex]);
			}
			else {
				System.err.println("Unknown argument "+currentIndex+": "+args[currentIndex]);
			}
			currentIndex++;
		}
		 System.out.println("you entered:\nsample file: "+chipOutFilename+"\ncontrol file: "+controlOutFilename+ "\nratio: "+ratio+"\n\n");

		//createSummary(ratio, outputDir, chipOutFilename, controlOutFilename, summaryFilename);
		createSummary(ratio, chipOutFilename, controlOutFilename, summaryFilename);
	}
}
